/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.mapper;


import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Param;
import org.springblade.manager.dto.WbsFormElementDTO;
import org.springblade.manager.dto.WbsFormElementDTO2;
import org.springblade.manager.entity.WbsFormElement;
import org.springblade.manager.vo.WbsFormElementVO;

import java.util.List;

/**
 * wbs表单元素 Mapper 接口
 *
 * @author liuyc
 * @since 2022-04-27
 */
public interface WbsFormElementMapper extends BaseMapper<WbsFormElement> {

    /**
     * 自定义分页
     *
     * @param page
     * @param wbsFormElement
     * @return
     */
    List<WbsFormElementVO> selectWbsFormElementPage(IPage page, WbsFormElementVO wbsFormElement);


    Integer createTable(@Param("sqlStr") String sqlStr, @Param("newTableName") String newTableName);

    int alterAddFiled(@Param("sqlStr") String sqlStr, @Param("tableName") String tableName);

    int deleteTableFiled(@Param("eKey") String eKey, @Param("tableName") String tableName);

    void updateFiledType(String initTableName, String eKey, String eType, Integer eLength);

    int judgeFiled(String eKey, String tableName);

    List<WbsFormElementDTO2> selectWbsTreeTableListByParentId(String parentId);

    void updateSortById(String id, Integer sort);

    WbsFormElementVO selectDetailById(Long id);
}
